/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "dialogcompanion.h"
#include "utilities.h"


static CompanionDialog *myobject;


void
companiondialog_on_okbutton_clicked  (GtkButton       *button,
                                      gpointer         user_data)
{
myobject->on_okbutton_clicked();
}


void
companiondialog_on_checkbutton2_toggled
                                        (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
myobject->on_checkbutton2_toggled();
}


CompanionDialog::CompanionDialog (Configuration *configuration)
{
  myobject = this;

  companiondialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (companiondialog), "Bibledit Windows Companions");
  gtk_window_set_position (GTK_WINDOW (companiondialog), GTK_WIN_POS_CENTER_ON_PARENT);
  gtk_window_set_modal (GTK_WINDOW (companiondialog), TRUE);

  dialog_vbox1 = GTK_DIALOG (companiondialog)->vbox;
  gtk_widget_show (dialog_vbox1);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

  label1 = gtk_label_new ("Bibledit can work together with a \"Bibledit Windows Companion\".\nThis companion gives Bibledit access to some Bible translation programs\nthat run on Windows or Wine. There can be two companions.");
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (vbox1), label1, FALSE, FALSE, 10);
  gtk_misc_set_alignment (GTK_MISC (label1), 0, 0.5);

  label2 = gtk_label_new ("One running on this computer. It communicates with Windows programs running on Wine.");
  gtk_widget_show (label2);
  gtk_box_pack_start (GTK_BOX (vbox1), label2, FALSE, FALSE, 10);
  gtk_misc_set_alignment (GTK_MISC (label2), 0, 0.5);

  checkbutton1 = gtk_check_button_new_with_mnemonic ("Use the companion on this computer");
  gtk_widget_show (checkbutton1);
  gtk_box_pack_start (GTK_BOX (vbox1), checkbutton1, FALSE, FALSE, 0);
  // Added to set the checkbutton.
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton1), configuration->companion_local_enabled);

  label3 = gtk_label_new ("Another one running on a virtual machine with Windows programs running there.");
  gtk_widget_show (label3);
  gtk_box_pack_start (GTK_BOX (vbox1), label3, FALSE, FALSE, 10);
  gtk_misc_set_alignment (GTK_MISC (label3), 0, 0.5);

  checkbutton2 = gtk_check_button_new_with_mnemonic ("Use the companion on the virtual machine");
  gtk_widget_show (checkbutton2);
  gtk_box_pack_start (GTK_BOX (vbox1), checkbutton2, FALSE, FALSE, 0);
  // Set the button active or not.
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton2), configuration->companion_remote_enabled);

  label4 = gtk_label_new ("Enter its IP address or hostname below.");
  gtk_widget_show (label4);
  gtk_box_pack_start (GTK_BOX (vbox1), label4, FALSE, FALSE, 10);
  gtk_misc_set_alignment (GTK_MISC (label4), 0, 0.5);

  entry = gtk_entry_new ();
  gtk_widget_show (entry);
  gtk_box_pack_start (GTK_BOX (vbox1), entry, FALSE, FALSE, 0);
  // Edited from Glade's code.
  gtk_entry_set_text (GTK_ENTRY (entry), configuration->companion_remote_name.c_str());
  gtk_entry_set_activates_default (GTK_ENTRY (entry), TRUE);

  dialog_action_area1 = GTK_DIALOG (companiondialog)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);

  cancelbutton = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (companiondialog), cancelbutton, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton, GTK_CAN_DEFAULT);

  okbutton = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (companiondialog), okbutton, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton, GTK_CAN_DEFAULT);

  g_signal_connect ((gpointer) okbutton, "clicked",
                    G_CALLBACK (companiondialog_on_okbutton_clicked),
                    NULL);
  g_signal_connect ((gpointer) checkbutton2, "toggled",
                    G_CALLBACK (companiondialog_on_checkbutton2_toggled),
                    NULL);

  gtk_widget_grab_focus (checkbutton1);
  gtk_widget_grab_default (okbutton);

  set_gui ();
}


CompanionDialog::~CompanionDialog ()
{
  gtk_widget_destroy (companiondialog);
}


int CompanionDialog::run ()
{
  return gtk_dialog_run (GTK_DIALOG (companiondialog));
}


void CompanionDialog::set_gui ()
{
  gtk_widget_set_sensitive (label4, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton2)));
  gtk_widget_set_sensitive (entry, gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton2)));
}


void CompanionDialog::on_okbutton_clicked()
{
  local = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton1));
  remote = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (checkbutton2));
  address = gtk_entry_get_text (GTK_ENTRY (entry));
}


void CompanionDialog::on_checkbutton2_toggled()
{
  set_gui();
}
