/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "dialogconvert2web.h"
#include "utilities.h"
#include "constants.h"
#include "htmlbrowser.h"
#include "html.h"
#include "scripture.h"
#include "glib.h"


static ConvertToWebDialog *myobject;


void
convert2webdialog_on_directorybutton_clicked
                                        (GtkButton       *button,
                                        gpointer         user_data)
{
  myobject->on_buttondirectory_clicked ();
}


void
convert2webdialog_on_convertbutton_clicked
                                        (GtkButton       *button,
                                        gpointer         user_data)
{
  myobject->on_buttonconvert_clicked ();
}


void
convert2webdialog_on_browsebutton_clicked
                                        (GtkButton       *button,
                                        gpointer         user_data)
{
  myobject->on_buttonbrowse_clicked ();
}


ConvertToWebDialog::ConvertToWebDialog (Configuration *configuration)
{
  myobject = this;
  converttowebdirectory = configuration->converttowebdirectory;

  // Store all the filenames to process.
  Scripture scripture (configuration->project);
  for (unsigned int i = 0; i < scripture.paths.size (); i++) {
    filenames.push_back (scripture.paths[i]);
  }

  convert2webdialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (convert2webdialog), "Convert a project to web files");
  gtk_window_set_position (GTK_WINDOW (convert2webdialog), GTK_WIN_POS_CENTER_ON_PARENT);
  gtk_window_set_modal (GTK_WINDOW (convert2webdialog), TRUE);

  dialog_vbox1 = GTK_DIALOG (convert2webdialog)->vbox;
  gtk_widget_show (dialog_vbox1);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

  label1 = gtk_label_new ("Converting a project to web files consists of the following steps.\n1. Indicating the directory where the web files are to be placed.\n   (this is not needed every time, only when there is a change)\n2. Converting the project.");
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (vbox1), label1, FALSE, FALSE, 5);
  gtk_misc_set_alignment (GTK_MISC (label1), 0.02, 0.5);

  label2 = gtk_label_new ("Indicate the directory where the web files are to be placed:");
  gtk_widget_show (label2);
  gtk_box_pack_start (GTK_BOX (vbox1), label2, FALSE, FALSE, 1);
  gtk_misc_set_alignment (GTK_MISC (label2), 0.02, 0.5);

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox1, TRUE, TRUE, 0);

  entry1 = gtk_entry_new ();
  gtk_widget_show (entry1);
  gtk_box_pack_start (GTK_BOX (hbox1), entry1, TRUE, TRUE, 0);
  gtk_editable_set_editable (GTK_EDITABLE (entry1), FALSE);

  directorybutton = gtk_button_new_with_mnemonic ("...");
  gtk_widget_show (directorybutton);
  gtk_box_pack_start (GTK_BOX (hbox1), directorybutton, FALSE, FALSE, 0);
  gtk_widget_set_size_request (directorybutton, 30, -1);

  hbox2 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox2);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox2, TRUE, TRUE, 8);

  convertbutton = gtk_button_new ();
  gtk_widget_show (convertbutton);
  gtk_box_pack_start (GTK_BOX (hbox2), convertbutton, FALSE, FALSE, 7);

  alignment1 = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_widget_show (alignment1);
  gtk_container_add (GTK_CONTAINER (convertbutton), alignment1);

  hbox3 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox3);
  gtk_container_add (GTK_CONTAINER (alignment1), hbox3);

  image1 = gtk_image_new_from_stock ("gtk-execute", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image1);
  gtk_box_pack_start (GTK_BOX (hbox3), image1, FALSE, FALSE, 0);

  label3 = gtk_label_new_with_mnemonic ("_Convert");
  gtk_widget_show (label3);
  gtk_box_pack_start (GTK_BOX (hbox3), label3, FALSE, FALSE, 0);

  browsebutton = gtk_button_new ();
  gtk_widget_show (browsebutton);
  gtk_box_pack_start (GTK_BOX (hbox2), browsebutton, FALSE, FALSE, 0);

  alignment2 = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_widget_show (alignment2);
  gtk_container_add (GTK_CONTAINER (browsebutton), alignment2);

  hbox4 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox4);
  gtk_container_add (GTK_CONTAINER (alignment2), hbox4);

  image2 = gtk_image_new_from_stock ("gtk-open", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image2);
  gtk_box_pack_start (GTK_BOX (hbox4), image2, FALSE, FALSE, 0);

  label4 = gtk_label_new_with_mnemonic ("_Open in browser");
  gtk_widget_show (label4);
  gtk_box_pack_start (GTK_BOX (hbox4), label4, FALSE, FALSE, 0);

  dialog_action_area1 = GTK_DIALOG (convert2webdialog)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);

  cancelbutton1 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton1);
  gtk_dialog_add_action_widget (GTK_DIALOG (convert2webdialog), cancelbutton1, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton1, GTK_CAN_DEFAULT);

  okbutton1 = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton1);
  gtk_dialog_add_action_widget (GTK_DIALOG (convert2webdialog), okbutton1, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton1, GTK_CAN_DEFAULT);

  g_signal_connect ((gpointer) directorybutton, "clicked",
                    G_CALLBACK (convert2webdialog_on_directorybutton_clicked),
                    NULL);
  g_signal_connect ((gpointer) convertbutton, "clicked",
                    G_CALLBACK (convert2webdialog_on_convertbutton_clicked),
                    NULL);
  g_signal_connect ((gpointer) browsebutton, "clicked",
                    G_CALLBACK (convert2webdialog_on_browsebutton_clicked),
                    NULL);

  gtk_widget_grab_focus (convertbutton);
  gtk_widget_grab_default (okbutton1);

  set_gui ();
}


ConvertToWebDialog::~ConvertToWebDialog ()
{
  gtk_widget_destroy (convert2webdialog);
}


int ConvertToWebDialog::run()
{
  return gtk_dialog_run (GTK_DIALOG (convert2webdialog));
}


void
ConvertToWebDialog::on_buttondirectory_clicked ()
{
  GtkWidget *file_selector;
  /* Create the selector */
  file_selector = gtk_file_selection_new ("Please choose a folder");
  // Prevent the user from selecting a file.
  gtk_widget_hide (GTK_FILE_SELECTION (file_selector)->file_list);
  gtk_file_selection_hide_fileop_buttons (GTK_FILE_SELECTION (file_selector));
  string s = converttowebdirectory + "/";
  gtk_file_selection_set_filename (GTK_FILE_SELECTION (file_selector), s.c_str());
  /* Display the dialog */
  gtk_widget_show (file_selector);
  int result = gtk_dialog_run (GTK_DIALOG (file_selector));
  string mydir = gtk_file_selection_get_filename (GTK_FILE_SELECTION (file_selector));
  gtk_widget_destroy (file_selector);
  // Handle the response:
  if (result == GTK_RESPONSE_OK)
    {                           
      converttowebdirectory = mydir;
      set_gui ();
    }
}


void
ConvertToWebDialog::on_buttonconvert_clicked ()
{
  HtmlConverter hc (0);
  hc.convert (filenames, converttowebdirectory);
}


void
ConvertToWebDialog::on_buttonbrowse_clicked ()
{
  htmlbrowser (g_build_filename (converttowebdirectory.c_str(), "index.html", NULL));
}


void
ConvertToWebDialog::set_gui ()
{
  // This sets the text in the dialog.
  gtk_entry_set_text (GTK_ENTRY (entry1), converttowebdirectory.c_str());
}
