/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include <glib.h>
#include "dialogfind.h"
#include "utilities.h"
#include "sqlite3_plus.h"
#include "bible.h"
#include "utilities.h"


static FindDialog *myobject;


void
finddialog_on_buttonfind_clicked ()
{
  myobject->on_buttonfind_clicked ();
}


FindDialog::FindDialog (const string& database, const string& search_in, const string& replace_in)
{
  myobject = this;
  mydatabase = database;

  finddialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (finddialog), "Find / Replace");
  gtk_window_set_position (GTK_WINDOW (finddialog),
                           GTK_WIN_POS_CENTER_ON_PARENT);
  gtk_window_set_modal (GTK_WINDOW (finddialog), TRUE);
  // Next one added to Glade's code.
  gtk_window_set_destroy_with_parent (GTK_WINDOW (finddialog), TRUE);

  dialog_vbox1 = GTK_DIALOG (finddialog)->vbox;
  gtk_widget_show (dialog_vbox1);

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), hbox1, TRUE, TRUE, 0);

  notebook1 = gtk_notebook_new ();
  gtk_widget_show (notebook1);
  gtk_box_pack_start (GTK_BOX (hbox1), notebook1, TRUE, TRUE, 2);

  hbox2 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox2);
  gtk_container_add (GTK_CONTAINER (notebook1), hbox2);

  label5 = gtk_label_new ("Search for");
  gtk_widget_show (label5);
  gtk_box_pack_start (GTK_BOX (hbox2), label5, FALSE, FALSE, 4);

  entry1 = gtk_entry_new ();
  // Next one has been added to Glade's code.
  gtk_entry_set_text (GTK_ENTRY (entry1), search_in.c_str ());
  gtk_widget_show (entry1);
  gtk_box_pack_start (GTK_BOX (hbox2), entry1, TRUE, TRUE, 0);
  gtk_entry_set_activates_default (GTK_ENTRY (entry1), TRUE);

  label1 = gtk_label_new ("Find");
  gtk_widget_show (label1);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1),
                              gtk_notebook_get_nth_page (GTK_NOTEBOOK
                                                         (notebook1), 0),
                              label1);

  table1 = gtk_table_new (2, 2, FALSE);
  gtk_widget_show (table1);
  gtk_container_add (GTK_CONTAINER (notebook1), table1);

  label7 = gtk_label_new ("Search for");
  gtk_widget_show (label7);
  gtk_table_attach (GTK_TABLE (table1), label7, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 4, 4);
  gtk_misc_set_alignment (GTK_MISC (label7), 1, 0.5);

  label8 = gtk_label_new ("Replace with");
  gtk_widget_show (label8);
  gtk_table_attach (GTK_TABLE (table1), label8, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 4, 4);
  gtk_misc_set_alignment (GTK_MISC (label8), 1, 0.5);

  entry2 = gtk_entry_new ();
  // Manually added code.
  gtk_entry_set_text (GTK_ENTRY (entry2), search_in.c_str ());
  gtk_widget_show (entry2);
  gtk_table_attach (GTK_TABLE (table1), entry2, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 2);
  gtk_entry_set_activates_default (GTK_ENTRY (entry2), TRUE);

  entry3 = gtk_entry_new ();
  // Manually added code.
  gtk_entry_set_text (GTK_ENTRY (entry3), replace_in.c_str ());
  gtk_widget_show (entry3);
  gtk_table_attach (GTK_TABLE (table1), entry3, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 2);
  gtk_entry_set_activates_default (GTK_ENTRY (entry3), TRUE);

  label6 = gtk_label_new ("Replace");
  gtk_widget_show (label6);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1),
                              gtk_notebook_get_nth_page (GTK_NOTEBOOK
                                                         (notebook1), 1),
                              label6);

  dialog_action_area1 = GTK_DIALOG (finddialog)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1),
                             GTK_BUTTONBOX_END);

  buttonfind = gtk_button_new_from_stock ("gtk-find");
  gtk_widget_show (buttonfind);
  gtk_dialog_add_action_widget (GTK_DIALOG (finddialog), buttonfind,
                                GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (buttonfind, GTK_CAN_DEFAULT);

  buttoncancel = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (buttoncancel);
  gtk_dialog_add_action_widget (GTK_DIALOG (finddialog), buttoncancel,
                                GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (buttoncancel, GTK_CAN_DEFAULT);

  // The next one modified from Glade's code.
  g_signal_connect ((gpointer) buttonfind, "clicked",
                    G_CALLBACK (finddialog_on_buttonfind_clicked), NULL);

  gtk_widget_grab_focus (entry1);
  gtk_widget_grab_default (buttonfind);

  // Next one added to Glade's code.
  gtk_dialog_set_default_response (GTK_DIALOG (finddialog), GTK_RESPONSE_OK);
}


FindDialog::~FindDialog ()
{
  gtk_widget_destroy (finddialog);
}


int FindDialog::run ()
{
  return gtk_dialog_run (GTK_DIALOG (finddialog));
}


void FindDialog::on_buttonfind_clicked ()
{
  try
  {
    replacing = (gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook1)) == 1);
    if (replacing)
      searchword = gtk_entry_get_text (GTK_ENTRY (entry2));
    else
      searchword = gtk_entry_get_text (GTK_ENTRY (entry1));
    replaceword = gtk_entry_get_text (GTK_ENTRY (entry3));
    // Set a flag if the text is still being indexed at present.
    is_indexing = program_is_running ("bibledit.wdb");
    // Connect to database.
    sqlite3::connection con (mydatabase.c_str ());
    // Get the string to search for. Note any apostrophies need to be doubled for SQLite.
    // We need to normalize the search expression, as prescribed, when comparing strings.
    ustring localsearchword;
    localsearchword = searchword;
    localsearchword = localsearchword.normalize ();
    localsearchword = localsearchword.casefold ();
    localsearchword = double_apostrophy (localsearchword);
    // Execute the SQL statement, and read the results.
    sqlite3::reader reader = con.executereader ("select book, chapter, verse from 'line_casefolded' where line glob ('*%s*');", localsearchword.c_str ());
    while (reader.read ())
      results.push_back (index_to_biblebook (reader.getint32 (0)) + " " + reader.getstring (1) + ":" + reader.getstring (2));
    // Close all
    reader.close ();
    con.close ();
  }
  catch (exception & ex)
  {
    cerr << "Error: " << ex.what () << endl;
  }
}
