/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "dialoglanguage.h"
#include "utilities.h"
#include "project.h"
#include "projectutils.h"
#include "projects.h"
#include "languageutils.h"
#include "language.h"
#include "languages.h"
#include "directories.h"


static LanguageDialog *myobject;


void
languagedialog_on_okbutton_clicked     (GtkButton       *button,
                                        gpointer         user_data)
{
  myobject->on_ok();
}


void
languagedialog_on_cancelbutton1_clicked
                                        (GtkButton       *button,
                                        gpointer         user_data)
{
  myobject->on_cancel();
}


void
languagedialog_on_languageentry_changed
                                        (GtkEditable     *editable,
                                        gpointer         user_data)
{
  myobject->set_gui();
}


LanguageDialog::LanguageDialog (const string & languagename)
{
  // Save variabeles.
  myobject = this;
  currentlanguagename = languagename;

  languagedialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (languagedialog), "Language properties");
  gtk_window_set_position (GTK_WINDOW (languagedialog), GTK_WIN_POS_CENTER_ON_PARENT);
  gtk_window_set_modal (GTK_WINDOW (languagedialog), TRUE);
  gtk_window_set_type_hint (GTK_WINDOW (languagedialog), GDK_WINDOW_TYPE_HINT_DIALOG);

  dialog_vbox1 = GTK_DIALOG (languagedialog)->vbox;
  gtk_widget_show (dialog_vbox1);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

  label6 = gtk_label_new ("Language:");
  gtk_widget_show (label6);
  gtk_box_pack_start (GTK_BOX (vbox1), label6, FALSE, FALSE, 2);
  gtk_misc_set_alignment (GTK_MISC (label6), 0, 0.5);

  languageentry = gtk_entry_new ();
  gtk_widget_show (languageentry);
  gtk_box_pack_start (GTK_BOX (vbox1), languageentry, FALSE, FALSE, 2);
  // We only allow the name to be changed when the language is not in use.
  bool editable = (languageutils_lang_used_by (currentlanguagename).size () == 0);
  gtk_editable_set_editable (GTK_EDITABLE (languageentry), editable);
  // Set name of language.
  gtk_entry_set_text (GTK_ENTRY (languageentry), languagename.c_str());
  gtk_entry_set_activates_default (GTK_ENTRY (languageentry), TRUE);

  messagelabel = gtk_label_new ("");
  gtk_widget_show (messagelabel);
  gtk_box_pack_start (GTK_BOX (vbox1), messagelabel, FALSE, FALSE, 2);
  gtk_misc_set_alignment (GTK_MISC (messagelabel), 0, 0.5);

  label5 = gtk_label_new ("Punctuation:");
  gtk_widget_show (label5);
  gtk_box_pack_start (GTK_BOX (vbox1), label5, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (label5), 0, 0.5);

  punctuationentry = gtk_entry_new ();
  gtk_widget_show (punctuationentry);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), punctuationentry, FALSE, FALSE, 0);
  // Get info about the language.
  Language language (g_build_filename (directories_get_languages ().c_str(), currentlanguagename.c_str(), NULL));
  gtk_entry_set_text (GTK_ENTRY (punctuationentry), language.punctuation.c_str());
  gtk_entry_set_activates_default (GTK_ENTRY (punctuationentry), TRUE);

  dialog_action_area1 = GTK_DIALOG (languagedialog)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);

  cancelbutton1 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton1);
  gtk_dialog_add_action_widget (GTK_DIALOG (languagedialog), cancelbutton1, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton1, GTK_CAN_DEFAULT);

  okbutton = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (languagedialog), okbutton, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton, GTK_CAN_DEFAULT);

  g_signal_connect ((gpointer) languageentry, "changed",
                    G_CALLBACK (languagedialog_on_languageentry_changed),
                    NULL);
  g_signal_connect ((gpointer) cancelbutton1, "clicked",
                    G_CALLBACK (languagedialog_on_cancelbutton1_clicked),
                    NULL);
  g_signal_connect ((gpointer) okbutton, "clicked",
                    G_CALLBACK (languagedialog_on_okbutton_clicked),
                    NULL);

  gtk_widget_grab_focus (punctuationentry);
  gtk_widget_grab_default (okbutton);

  // Handle gui.
  set_gui();
}


LanguageDialog::~LanguageDialog ()
{
  gtk_widget_destroy (languagedialog);
}


int LanguageDialog::run ()
{
  return gtk_dialog_run (GTK_DIALOG (languagedialog));
}


void LanguageDialog::set_gui ()
{
  bool sensitive = true;
  newlanguagename = gtk_entry_get_text (GTK_ENTRY (languageentry));
  newlanguagename = trim (newlanguagename);
  // Reject "New Language" as a name.
  if (newlanguagename == NEW_LANGUAGE)
    {
      sensitive = false;
      gtk_label_set_text (GTK_LABEL (messagelabel), "(the name of the language should be changed)");
    }
  // Reject empty names.
  if (newlanguagename.empty ())
    {
      sensitive = false;
      gtk_label_set_text (GTK_LABEL (messagelabel), "(the language needs a name)");
    }
  // Reject names that already exist.
  if (currentlanguagename != newlanguagename)
    {
      Languages languages (directories_get_languages ());
      for (unsigned int i = 0; i < languages.languages.size (); i++)
        if (newlanguagename == languages.languages[i])
          {
            sensitive = false;
            gtk_label_set_text (GTK_LABEL (messagelabel), "(a language with this name already exists)");
          }
    }
  // Deal with the sensitivity of the dialog.
  gtk_widget_set_sensitive (okbutton, sensitive);
  if (sensitive)
    gtk_label_set_text (GTK_LABEL (messagelabel), "");
}


void
LanguageDialog::on_ok ()
{
  // If the name changed, carry this change out.
  if (currentlanguagename != newlanguagename)
    {
      string oldname = (g_build_filename (directories_get_languages ().c_str(), currentlanguagename.c_str(), NULL));
      string newname = (g_build_filename (directories_get_languages ().c_str(), newlanguagename.c_str(), NULL));
      string command = "mv \"";
      command.append (g_strescape (oldname.c_str(), NULL));
      command.append ("\" \"");
      command.append (g_strescape (newname.c_str(), NULL));
      command.append ("\"");
      system (command.c_str());
    }
  // Save the settings belonging to the language.
  Language language (g_build_filename (directories_get_languages ().c_str(), newlanguagename.c_str(), NULL));
  language.punctuation = gtk_entry_get_text (GTK_ENTRY (punctuationentry));
}


void
LanguageDialog::on_cancel ()
{
  // Remove the "New Language", because it was created but not used.
  string delname = g_build_filename (directories_get_languages ().c_str(), NEW_LANGUAGE, NULL);
  string command = "rm -rf '" + delname + "'";
  system (command.c_str());
}


