/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include <glib.h>
#include "dialogreplace.h"
#include "utilities.h"
#include "bible.h"
#include "book.h"
#include "chapter.h"
#include "usfm.h"
#include "usfmtools.h"


static ReplaceDialog *myobject;


void
replacedialog_on_cancelbutton_clicked (GtkButton * button, gpointer user_data)
{
  myobject->on_cancelbutton_clicked ();
}


void
replacedialog_on_yesbutton_clicked (GtkButton * button, gpointer user_data)
{
  myobject->on_yesbutton_clicked ();
}


void
replacedialog_on_nobutton_clicked (GtkButton * button, gpointer user_data)
{
  myobject->on_nobutton_clicked ();
}


void
replacedialog_on_allbutton_clicked (GtkButton * button, gpointer user_data)
{
  myobject->on_allbutton_clicked ();
}


ReplaceDialog::ReplaceDialog (const string& projectname,
                              const vector <string>& references_in,
                              const ustring& searchword,
                              const ustring& replaceword):
scripture (projectname)
{
  myobject = this;
  searchfor = searchword;
  replacewith = replaceword;

  replacedialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (replacedialog), "Replace");
  // Next one added to Glade's code.
  gtk_window_set_destroy_with_parent (GTK_WINDOW (replacedialog), TRUE);

  dialog_vbox1 = GTK_DIALOG (replacedialog)->vbox;
  gtk_widget_show (dialog_vbox1);

  referencelabel = gtk_label_new ("referencelabel");
  gtk_widget_show (referencelabel);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), referencelabel, FALSE, FALSE,
                      4);
  gtk_misc_set_alignment (GTK_MISC (referencelabel), 0, 0.5);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

  label1 = gtk_label_new ("Do you wish to replace this text: ...");
  gtk_widget_show (label1);
  gtk_box_pack_start (GTK_BOX (vbox1), label1, FALSE, FALSE, 4);

  scrolledwindow4 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow4);
  gtk_box_pack_start (GTK_BOX (vbox1), scrolledwindow4, TRUE, TRUE, 0);

  textview1 = gtk_text_view_new ();
  gtk_widget_show (textview1);
  gtk_container_add (GTK_CONTAINER (scrolledwindow4), textview1);
  gtk_widget_set_size_request (textview1, 500, 100);
  gtk_text_view_set_editable (GTK_TEXT_VIEW (textview1), FALSE);
  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (textview1), GTK_WRAP_WORD);
  gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (textview1), FALSE);

  label2 = gtk_label_new ("... with this text? It can be edited also.");
  gtk_widget_show (label2);
  gtk_box_pack_start (GTK_BOX (vbox1), label2, FALSE, FALSE, 4);

  scrolledwindow5 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow5);
  gtk_box_pack_start (GTK_BOX (vbox1), scrolledwindow5, TRUE, TRUE, 0);

  textview2 = gtk_text_view_new ();
  gtk_widget_show (textview2);
  gtk_container_add (GTK_CONTAINER (scrolledwindow5), textview2);
  gtk_widget_set_size_request (textview2, 500, 100);
  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (textview2), GTK_WRAP_WORD);

  hbox3 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox3);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox3, FALSE, FALSE, 5);

  yesbutton = gtk_button_new_from_stock ("gtk-yes");
  gtk_widget_show (yesbutton);
  gtk_box_pack_start (GTK_BOX (hbox3), yesbutton, FALSE, FALSE, 4);
  gtk_widget_set_size_request (yesbutton, 80, -1);
  GTK_WIDGET_SET_FLAGS (yesbutton, GTK_CAN_DEFAULT);

  nobutton = gtk_button_new_from_stock ("gtk-no");
  gtk_widget_show (nobutton);
  gtk_box_pack_start (GTK_BOX (hbox3), nobutton, FALSE, FALSE, 4);
  gtk_widget_set_size_request (nobutton, 80, -1);

  allbutton = gtk_button_new ();
  gtk_widget_show (allbutton);
  gtk_box_pack_start (GTK_BOX (hbox3), allbutton, FALSE, FALSE, 4);
  gtk_widget_set_size_request (allbutton, 80, -1);

  alignment1 = gtk_alignment_new (0.5, 0.5, 0, 0);
  gtk_widget_show (alignment1);
  gtk_container_add (GTK_CONTAINER (allbutton), alignment1);

  hbox4 = gtk_hbox_new (FALSE, 2);
  gtk_widget_show (hbox4);
  gtk_container_add (GTK_CONTAINER (alignment1), hbox4);

  image1 = gtk_image_new_from_stock ("gtk-dialog-warning", GTK_ICON_SIZE_BUTTON);
  gtk_widget_show (image1);
  gtk_box_pack_start (GTK_BOX (hbox4), image1, FALSE, FALSE, 0);

  label3 = gtk_label_new_with_mnemonic ("_All");
  gtk_widget_show (label3);
  gtk_box_pack_start (GTK_BOX (hbox4), label3, FALSE, FALSE, 0);

  cancelbutton = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton);
  gtk_box_pack_start (GTK_BOX (hbox3), cancelbutton, FALSE, FALSE, 4);
  gtk_widget_set_size_request (cancelbutton, 80, -1);

  dialog_action_area1 = GTK_DIALOG (replacedialog)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1),
                             GTK_BUTTONBOX_END);

  gtk_widget_grab_focus (yesbutton);
  gtk_widget_grab_default (yesbutton);

  g_signal_connect ((gpointer) yesbutton, "clicked",
                    G_CALLBACK (replacedialog_on_yesbutton_clicked), NULL);
  g_signal_connect ((gpointer) nobutton, "clicked",
                    G_CALLBACK (replacedialog_on_nobutton_clicked), NULL);
  g_signal_connect ((gpointer) allbutton, "clicked",
                    G_CALLBACK (replacedialog_on_allbutton_clicked), NULL);
  g_signal_connect ((gpointer) cancelbutton, "clicked",
                    G_CALLBACK (replacedialog_on_cancelbutton_clicked), NULL);

  // Next one added to Glade's code.
  gtk_dialog_set_default_response (GTK_DIALOG (replacedialog),
                                   GTK_RESPONSE_OK);

  // Processing code.
  for (unsigned int i = 0; i < references_in.size (); i++)
    references.push_back (references_in[i]);
  referencepointer = 0;
  set_gui ();
}


ReplaceDialog::~ReplaceDialog ()
{
  gtk_widget_destroy (replacedialog);
}


int
ReplaceDialog::run ()
{
  return gtk_dialog_run (GTK_DIALOG (replacedialog));
}


void
ReplaceDialog::on_yesbutton_clicked ()
{
  accept_change();
  referencepointer++;
  set_gui ();
  if (referencepointer >= references.size ())
    on_cancelbutton_clicked ();
}


void
ReplaceDialog::on_nobutton_clicked ()
{
  referencepointer++;
  set_gui ();
  if (referencepointer >= references.size ())
    on_cancelbutton_clicked ();
}


void
ReplaceDialog::on_allbutton_clicked ()
{
  while (referencepointer < references.size ()) {
    on_yesbutton_clicked ();
  }
}


void
ReplaceDialog::on_cancelbutton_clicked ()
{
  gtk_dialog_response (GTK_DIALOG (replacedialog), GTK_RESPONSE_NONE);
}


void
ReplaceDialog::set_gui ()
/*
 * The GUI is filled with all data needed.
 */
{
  if (referencepointer < references.size ())
    {
      // Show reference.
      gtk_label_set_text (GTK_LABEL (referencelabel),
                          references[referencepointer].c_str ());
      // Get the reference decoded, and get the book number.
      string book, chapter, verse;
      decode_reference (references[referencepointer], book, chapter, verse);
      int book_index = scripture.get_index_of_book (book);
      // Define two textbuffers to work with.
      GtkTextBuffer *buffer1;
      buffer1 = gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview1));
      GtkTextBuffer *buffer2;
      buffer2 = gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview2));
      // Book exists?
      if (book_index >= 0)
        {
          // Open the book, get the chapter and the verse text.
          Book book (scripture.paths[book_index], false, false);
          vector < ustring > lines;
          book.get_chapter (convert_to_int (chapter), lines);
          Chapter chapter (lines);
          vector < ustring > verses;
          chapter.get_verse (convert_to_int (verse), verses);
          // Clear the two textviews.
          gtk_text_buffer_set_text (buffer1, "", -1);
          gtk_text_buffer_set_text (buffer2, "", -1);
          // Searching is not case-sensitive.
          ustring searchword = searchfor.casefold();
          /*
           * When both the search and replace strings are lower case,
           * then any word to be replaced, when it starts with a capital
           * will be replaced starting with a capital again.
           */
          bool bothlowercase;
          bothlowercase = ((searchfor == searchfor.lowercase()) && (replacewith == replacewith.lowercase()));
          // Usfm's are needed later on.
          Usfm usfm(0);
          // As a verse may consist of more lines, deal with each.
          for (unsigned int i = 0; i < verses.size (); i++) {
            // Fill original text.
            ustring s (verses[i]);
            gtk_text_buffer_insert_at_cursor (buffer1, s.c_str (), -1);
            gtk_text_buffer_insert_at_cursor (buffer1, "\n", -1);
            // Start producing modified text.
            ustring s2 (s.casefold());
            ustring sfm (verses[i]);
            // Replace the text, except in the \id line.
            if (!usfm.is_id(usfm_extract_marker(sfm))) {
              unsigned int offposition = s2.find (searchword);
              while (offposition != string::npos) {
                ustring firstchar = s.substr(offposition, 1);
                bool makeupper = false;
                if (firstchar == firstchar.uppercase())
                  if (bothlowercase)
                    makeupper = true;
                s.replace(offposition, searchword.length(), replacewith);
                if (makeupper) {
                  ustring s3 = s.substr(offposition, 1);
                  s3 = s3.uppercase();
                  s.replace (offposition, 1, s3);
                }
                s2 = s.casefold();
                offposition = s2.find (searchword, offposition + replacewith.length());
              }
            }
            // Insert modified text into the second buffer.
            gtk_text_buffer_insert_at_cursor (buffer2, s.c_str (), -1);
            gtk_text_buffer_insert_at_cursor (buffer2, "\n", -1);
          }
        }
    }
}


void 
ReplaceDialog::accept_change ()
/*
 * This function accepts the change from the second textview
 * and saves it into the book.
 */
{
  // Decode the reference into book, chapter and verse.
  string book, chapter, verse;
  decode_reference (references[referencepointer], book, chapter, verse);
  int book_index = scripture.get_index_of_book (book);
  GtkTextBuffer *buffer;
  buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview2));
  // Put the modified text back into the book.
  if (book_index >= 0) {
    Book book (scripture.paths[book_index], false, false);
    vector <ustring> lines;
    book.get_chapter (convert_to_int (chapter), lines);
    Chapter chapter (lines);
    vector <ustring> verses;
    chapter.get_verse (convert_to_int (verse), verses);
    verses.clear();
    int linecount = gtk_text_buffer_get_line_count (GTK_TEXT_BUFFER (buffer));
    for (int i = 0; i < linecount; i++) {
      GtkTextIter begin;
      GtkTextIter end;
      gtk_text_buffer_get_iter_at_line (GTK_TEXT_BUFFER (buffer), &begin, i);
      gtk_text_buffer_get_iter_at_line (GTK_TEXT_BUFFER (buffer), &end, i + 1);
      gtk_text_iter_backward_char (&end);
      ustring s;
      s = gtk_text_buffer_get_text (GTK_TEXT_BUFFER (buffer), &begin, &end, false);
      s = trim(s);
      if (s.length()) verses.push_back(s);
    }
    // Save the modified text.
    chapter.set_verse (verses);
    book.set_chapter (chapter.lines);
    book.save();
  }
}
