/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "utilities.h"
#include "dialogshowscript.h"
#include <glib.h>


static ShowScriptDialog *myobject;


bool show_script_dialog_scroll ()
{
  return myobject->scroll();
}

ShowScriptDialog::ShowScriptDialog (const string& filename)
{
  myobject = this;

  showscriptdialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (showscriptdialog), "Contents");
  gtk_window_set_position (GTK_WINDOW (showscriptdialog), GTK_WIN_POS_CENTER_ON_PARENT);
  gtk_window_set_modal (GTK_WINDOW (showscriptdialog), TRUE);
  gtk_window_set_default_size (GTK_WINDOW (showscriptdialog), 640, 480);
  gtk_window_set_type_hint (GTK_WINDOW (showscriptdialog), GDK_WINDOW_TYPE_HINT_DIALOG);

  dialog_vbox1 = GTK_DIALOG (showscriptdialog)->vbox;
  gtk_widget_show (dialog_vbox1);

  scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), scrolledwindow1, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  textview1 = gtk_text_view_new ();
  gtk_widget_show (textview1);
  gtk_container_add (GTK_CONTAINER (scrolledwindow1), textview1);
  gtk_text_view_set_editable (GTK_TEXT_VIEW (textview1), FALSE);
  // Insert the text from the file.
  ReadText rt (filename, true);
  for (unsigned int i = 0; i < rt.lines.size (); i++)
    {
      gtk_text_buffer_insert_at_cursor (gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview1)), rt.lines[i].c_str(), -1);
      gtk_text_buffer_insert_at_cursor (gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview1)), "\n", -1);
    }

  dialog_action_area1 = GTK_DIALOG (showscriptdialog)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);

  cancelbutton = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (showscriptdialog), cancelbutton, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton, GTK_CAN_DEFAULT);

  gtk_widget_grab_focus (textview1);
  gtk_widget_grab_default (cancelbutton);

  // Scroll to end;
  g_idle_add (GSourceFunc (show_script_dialog_scroll), NULL);
}


ShowScriptDialog::~ShowScriptDialog ()
{
  gtk_widget_destroy (showscriptdialog);
}


int ShowScriptDialog::run()
{
  return gtk_dialog_run (GTK_DIALOG (showscriptdialog));
}


bool
ShowScriptDialog::scroll ()
{
  // Scroll the view to the end
  GtkTextIter end;
  gtk_text_buffer_get_end_iter (gtk_text_view_get_buffer (GTK_TEXT_VIEW (textview1)), &end);
  gtk_text_view_scroll_to_iter (GTK_TEXT_VIEW (textview1), &end, 0, true, 0, 0);
  return false;
}
