/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include <glib.h>
#include "dialogstylesheet.h"
#include "utilities.h"
#include "project.h"
#include "projectutils.h"
#include "projects.h"
#include "stylesheetutils.h"
#include "stylesheet.h"
#include "stylesheets.h"
#include "directories.h"


static StylesheetDialog *myobject;


void
stylesheetdialog_on_stylesheetentry_changed
                                        (GtkEditable     *editable,
                                        gpointer         user_data)
{
  myobject->set_gui();
}


void
stylesheetdialog_on_cancelbutton1_clicked
                                        (GtkButton       *button,
                                        gpointer         user_data)
{
  myobject->on_cancel();
}


void
stylesheetdialog_on_okbutton_clicked   (GtkButton       *button,
                                        gpointer         user_data)
{
  myobject->on_ok();
}


StylesheetDialog::StylesheetDialog (const string & stylesheetname)
{
  // Save variabeles.
  myobject = this;
  currentstylesheetname = stylesheetname;

  stylesheetdialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (stylesheetdialog), "Stylesheet properties");
  gtk_window_set_position (GTK_WINDOW (stylesheetdialog), GTK_WIN_POS_CENTER_ON_PARENT);
  gtk_window_set_modal (GTK_WINDOW (stylesheetdialog), TRUE);
  gtk_window_set_type_hint (GTK_WINDOW (stylesheetdialog), GDK_WINDOW_TYPE_HINT_DIALOG);

  dialog_vbox1 = GTK_DIALOG (stylesheetdialog)->vbox;
  gtk_widget_show (dialog_vbox1);

  label6 = gtk_label_new ("Stylesheet:");
  gtk_widget_show (label6);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), label6, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (label6), 0, 0.5);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

  stylesheetentry = gtk_entry_new ();
  gtk_widget_show (stylesheetentry);
  gtk_box_pack_start (GTK_BOX (vbox1), stylesheetentry, FALSE, FALSE, 2);
  // We only allow the name to be changed when the stylesheet is not in use.
  bool editable = (stylesheetutils_lang_used_by (currentstylesheetname).size () == 0);
  gtk_editable_set_editable (GTK_EDITABLE (stylesheetentry), editable);
  // Stylesheet name.
  gtk_entry_set_text (GTK_ENTRY (stylesheetentry), currentstylesheetname.c_str());
  gtk_entry_set_activates_default (GTK_ENTRY (stylesheetentry), TRUE);

  messagelabel = gtk_label_new ("");
  gtk_widget_show (messagelabel);
  gtk_box_pack_start (GTK_BOX (vbox1), messagelabel, FALSE, FALSE, 2);
  gtk_misc_set_alignment (GTK_MISC (messagelabel), 0, 0.5);

  dialog_action_area1 = GTK_DIALOG (stylesheetdialog)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);

  cancelbutton1 = gtk_button_new_from_stock ("gtk-cancel");
  gtk_widget_show (cancelbutton1);
  gtk_dialog_add_action_widget (GTK_DIALOG (stylesheetdialog), cancelbutton1, GTK_RESPONSE_CANCEL);
  GTK_WIDGET_SET_FLAGS (cancelbutton1, GTK_CAN_DEFAULT);

  okbutton = gtk_button_new_from_stock ("gtk-ok");
  gtk_widget_show (okbutton);
  gtk_dialog_add_action_widget (GTK_DIALOG (stylesheetdialog), okbutton, GTK_RESPONSE_OK);
  GTK_WIDGET_SET_FLAGS (okbutton, GTK_CAN_DEFAULT);

  g_signal_connect ((gpointer) stylesheetentry, "changed",
                    G_CALLBACK (stylesheetdialog_on_stylesheetentry_changed),
                    NULL);
  g_signal_connect ((gpointer) cancelbutton1, "clicked",
                    G_CALLBACK (stylesheetdialog_on_cancelbutton1_clicked),
                    NULL);
  g_signal_connect ((gpointer) okbutton, "clicked",
                    G_CALLBACK (stylesheetdialog_on_okbutton_clicked),
                    NULL);

  gtk_widget_grab_default (okbutton);

  // Get info about the stylesheet.
  Stylesheet stylesheet (g_build_filename (directories_get_stylesheets ().c_str(), currentstylesheetname.c_str(), NULL));

  // Handle gui.
  set_gui();
}


StylesheetDialog::~StylesheetDialog ()
{
  gtk_widget_destroy (stylesheetdialog);
}


int StylesheetDialog::run ()
{
  return gtk_dialog_run (GTK_DIALOG (stylesheetdialog));
}


void StylesheetDialog::set_gui ()
{
  bool sensitive = true;
  newstylesheetname = gtk_entry_get_text (GTK_ENTRY (stylesheetentry));
  newstylesheetname = trim (newstylesheetname);
  // Reject "New Stylesheet" as a name.
  if (newstylesheetname == NEW_STYLESHEET)
    {
      sensitive = false;
      gtk_label_set_text (GTK_LABEL (messagelabel), "(the name of the stylesheet should be changed)");
    }
  // Reject empty names.
  if (newstylesheetname.empty ())
    {
      sensitive = false;
      gtk_label_set_text (GTK_LABEL (messagelabel), "(the stylesheet needs a name)");
    }
  // Reject names that already exist.
  if (currentstylesheetname != newstylesheetname)
    {
      Stylesheets stylesheets (directories_get_stylesheets ());
      for (unsigned int i = 0; i < stylesheets.stylesheets.size (); i++)
        if (newstylesheetname == stylesheets.stylesheets[i])
          {
            sensitive = false;
            gtk_label_set_text (GTK_LABEL (messagelabel), "(a stylesheet with this name already exists)");
          }
    }
  // Deal with the sensitivity of the dialog.
  gtk_widget_set_sensitive (okbutton, sensitive);
  if (sensitive) gtk_label_set_text (GTK_LABEL (messagelabel), "");
}


void StylesheetDialog::on_ok ()
{
  // If the name changed, carry this change out.
  if (currentstylesheetname != newstylesheetname)
    {
      string oldname = (g_build_filename (directories_get_stylesheets ().c_str(), currentstylesheetname.c_str(), NULL));
      string newname = (g_build_filename (directories_get_stylesheets ().c_str(), newstylesheetname.c_str(), NULL));
      string command = "mv \"";
      command.append (g_strescape (oldname.c_str(), NULL));
      command.append ("\" \"");
      command.append (g_strescape (newname.c_str(), NULL));
      command.append ("\"");
      system (command.c_str());
    }
  // Save the settings belonging to the stylesheet.
  Stylesheet stylesheet (g_build_filename (directories_get_stylesheets ().c_str(), newstylesheetname.c_str(), NULL));
  //  stylesheet.punctuation = entrypunctuation.get_text (); To be removed later, but left here as a template.
}


void StylesheetDialog::on_cancel ()
{
  // Remove the "New Stylesheet", because it was created but not used..
  string delname = (g_build_filename (directories_get_stylesheets ().c_str(), NEW_STYLESHEET, NULL));
  string command = "rm -rf '" + delname + "'";
  system (command.c_str());
}

