/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include <gtk/gtk.h>
#include "dialogstylesheets.h"
#include "utilities.h"
#include "project.h"
#include "projectutils.h"
#include "projects.h"
#include "stylesheets.h"
#include "stylesheetutils.h"
#include "dialogstylesheet.h"
#include "directories.h"


static StylesheetsDialog *myobject;


void
stylesheetsdialog_on_newbutton_clicked (GtkButton       *button,
                                        gpointer         user_data)
{
  myobject->on_new();
}


void
stylesheetsdialog_on_propertiesbutton_clicked
                                        (GtkButton       *button,
                                        gpointer         user_data)
{
  myobject->on_edit();
}


void
stylesheetsdialog_on_deletebutton_clicked (GtkButton       *button,
                                        gpointer         user_data)
{
  myobject->on_delete();
}

StylesheetsDialog::StylesheetsDialog (int dummy)
{
  myobject = this;

  stylesheetsdialog = gtk_dialog_new ();
  gtk_window_set_title (GTK_WINDOW (stylesheetsdialog), "Stylesheets");
  gtk_window_set_position (GTK_WINDOW (stylesheetsdialog), GTK_WIN_POS_CENTER_ON_PARENT);
  gtk_window_set_modal (GTK_WINDOW (stylesheetsdialog), TRUE);

  dialog_vbox1 = GTK_DIALOG (stylesheetsdialog)->vbox;
  gtk_widget_show (dialog_vbox1);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), vbox1, TRUE, TRUE, 0);

  combo1 = gtk_combo_new ();
  g_object_set_data (G_OBJECT (GTK_COMBO (combo1)->popwin),
                     "GladeParentKey", combo1);
  gtk_widget_show (combo1);
  gtk_box_pack_start (GTK_BOX (vbox1), combo1, FALSE, FALSE, 0);

  combo_entry1 = GTK_COMBO (combo1)->entry;
  gtk_widget_show (combo_entry1);
  gtk_editable_set_editable (GTK_EDITABLE (combo_entry1), FALSE);
  gtk_entry_set_text (GTK_ENTRY (combo_entry1), "Item 1");

  // Combo box with all stylesheets.
  fillcombo ("");

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox1, TRUE, TRUE, 2);

  newbutton = gtk_button_new_from_stock ("gtk-new");
  gtk_widget_show (newbutton);
  gtk_box_pack_start (GTK_BOX (hbox1), newbutton, FALSE, FALSE, 0);
  gtk_widget_set_size_request (newbutton, 90, -1);

  propertiesbutton = gtk_button_new_from_stock ("gtk-properties");
  gtk_widget_show (propertiesbutton);
  gtk_box_pack_start (GTK_BOX (hbox1), propertiesbutton, FALSE, FALSE, 0);
  gtk_widget_set_size_request (propertiesbutton, 90, -1);

  deletebutton = gtk_button_new_from_stock ("gtk-delete");
  gtk_widget_show (deletebutton);
  gtk_box_pack_start (GTK_BOX (hbox1), deletebutton, FALSE, FALSE, 0);
  gtk_widget_set_size_request (deletebutton, 90, -1);

  dialog_action_area1 = GTK_DIALOG (stylesheetsdialog)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);

  closebutton1 = gtk_button_new_from_stock ("gtk-close");
  gtk_widget_show (closebutton1);
  gtk_dialog_add_action_widget (GTK_DIALOG (stylesheetsdialog), closebutton1, GTK_RESPONSE_CLOSE);
  GTK_WIDGET_SET_FLAGS (closebutton1, GTK_CAN_DEFAULT);

  g_signal_connect ((gpointer) newbutton, "clicked",
                    G_CALLBACK (stylesheetsdialog_on_newbutton_clicked),
                    NULL);
  g_signal_connect ((gpointer) propertiesbutton, "clicked",
                    G_CALLBACK (stylesheetsdialog_on_propertiesbutton_clicked),
                    NULL);
  g_signal_connect ((gpointer) deletebutton, "clicked",
                    G_CALLBACK (stylesheetsdialog_on_deletebutton_clicked),
                    NULL);

  gtk_widget_grab_focus (closebutton1);
  gtk_widget_grab_default (closebutton1);

}


StylesheetsDialog::~StylesheetsDialog ()
{
  gtk_widget_destroy (stylesheetsdialog);
}


int StylesheetsDialog::run()
{
  return gtk_dialog_run (GTK_DIALOG (stylesheetsdialog));
}


void StylesheetsDialog::on_new ()
{
  stylesheetutils_new ("");
  StylesheetDialog dialog (NEW_STYLESHEET);
  if (dialog.run () == GTK_RESPONSE_OK)
    fillcombo (dialog.newstylesheetname);
}


void StylesheetsDialog::on_edit ()
{
  string stylesheet = gtk_entry_get_text (GTK_ENTRY (combo_entry1));
  StylesheetDialog dialog (stylesheet);
  if (dialog.run () == GTK_RESPONSE_YES)
    fillcombo ("");
}


void StylesheetsDialog::on_delete ()
{
  /*
     Delete the stylesheet, but only when it is not in use by any project.
   */
  // See if the stylesheet is in use.
  string stylesheet = gtk_entry_get_text (GTK_ENTRY (combo_entry1));
  vector < string > projects_using_this_stylesheet =
    stylesheetutils_lang_used_by (stylesheet);
  if (projects_using_this_stylesheet.size ()) {
    // In use: display info about it.
    string message = "This stylesheet could not be deleted, because it is in use by the following project(s):";
    for (unsigned int i = 0; i < projects_using_this_stylesheet.size (); i++)
      message.append ("\n- " + projects_using_this_stylesheet[i]);
    GtkWidget *dialog = gtk_message_dialog_new (GTK_WINDOW (stylesheetsdialog),
                                                GTK_DIALOG_DESTROY_WITH_PARENT,
                                                GTK_MESSAGE_ERROR,
                                                GTK_BUTTONS_OK,
                                                message.c_str ());
    gtk_dialog_run (GTK_DIALOG (dialog));
    gtk_widget_destroy (dialog);
  } else {
    // Not in use: delete it.
    string directory = g_build_filename (directories_get_stylesheets ().c_str(), gtk_entry_get_text (GTK_ENTRY (combo_entry1)), NULL);
    string command = "rm -rf \"";
    command.append (g_strescape (directory.c_str(), NULL));
    command.append ("\"");
    system (command.c_str());
    fillcombo ("");
  }
}


void
StylesheetsDialog::fillcombo (const string & active_stylesheet)
{
  Stylesheets stylesheets (directories_get_stylesheets ());
  if (stylesheets.stylesheets.size () > 0) {
    GList *combo1_items = NULL;
    for (unsigned int i = 0; i < stylesheets.stylesheets.size(); i++) {
      combo1_items = g_list_append (combo1_items, (gpointer) stylesheets.stylesheets[i].c_str());
    }
    gtk_combo_set_popdown_strings (GTK_COMBO (combo1), combo1_items);
    g_list_free (combo1_items);
    if (active_stylesheet.empty ())
      gtk_entry_set_text (GTK_ENTRY (combo_entry1), stylesheets.stylesheets[0].c_str());
    else
      gtk_entry_set_text (GTK_ENTRY (combo_entry1), active_stylesheet.c_str());
  }
}
