/*
** Copyright (C) 2003 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/

#ifndef INCLUDED_MAINWINDOW_H
#define INCLUDED_MAINWINDOW_H


#include <gtk/gtk.h>
#include "configuration.h"
#include "track.h"
#include "book.h"
//#include "backup.h"
#include "undo.h"
#include "windowscompanionlocal.h"
#include "windowscompanionremote.h"
#include "scripture.h"
#include "ustring.h"


class MainWindow {
public:
  MainWindow ();
  virtual ~MainWindow ();
  virtual int run();
  GtkWidget *mainwindow;
protected:
  // Widgets from Glade
  GtkWidget *vbox1;
  GtkWidget *menubar1;
  /* IMPORTANT If more menu items are added see whether they should be made 
   * in(sensitive) depending on whether a project has been opened. */
  GtkWidget *menuitem_file;
  GtkWidget *menuitem_file_menu;
  GtkWidget *new1;
  GtkWidget *open1;
  GtkWidget *close1;
  GtkWidget *delete1;
  GtkWidget *properties1;
  GtkWidget *import1;
  GtkWidget *image93;
  GtkWidget *separator3;
  GtkWidget *open_references1;
  GtkWidget *image94;
  GtkWidget *separatormenuitem1;
  GtkWidget *quit1;
  GtkWidget *menuitem_edit;
  GtkWidget *menuitem_edit_menu;
  GtkWidget *cut1;
  GtkWidget *copy1;
  GtkWidget *paste1;
  GtkWidget *separator2;
  GtkWidget *undo1;
  GtkWidget *image295;
  GtkWidget *redo1;
  GtkWidget *image296;
  GtkWidget *separator4;
  GtkWidget *find1;
  GtkWidget *menuitem_view;
  GtkWidget *menuitem_view_menu;
  GtkWidget *font1;
  GtkWidget *menuitem_goto;
  GtkWidget *menuitem_goto_menu;
  GtkWidget *next_verse1;
  GtkWidget *image95;
  GtkWidget *previous_verse1;
  GtkWidget *image96;
  GtkWidget *next_chapter1;
  GtkWidget *image97;
  GtkWidget *previous_chapter1;
  GtkWidget *image98;
  GtkWidget *next_book1;
  GtkWidget *image99;
  GtkWidget *previous_book1;
  GtkWidget *image100;
  GtkWidget *reference1;
  GtkWidget *image101;
  GtkWidget *menuitem_tools;
  GtkWidget *menuitem_tools_menu;
  GtkWidget *convert_project_to_web_pages1;
  GtkWidget *image102;
  GtkWidget *menuitem_preferences;
  GtkWidget *menuitem_preferences_menu;
  GtkWidget *bibledit_windows_companion1;
  GtkWidget *image103;
  GtkWidget *languages1;
  GtkWidget *image104;
  GtkWidget *stylesheets1;
  GtkWidget *image105;
  GtkWidget *menuitem_diagnostics;
  GtkWidget *menuitem_diagnostics_menu;
  GtkWidget *view_system_log1;
  GtkWidget *image106;
  GtkWidget *menuitem_help;
  GtkWidget *menuitem_help_menu;
  GtkWidget *help1;
  GtkWidget *about1;
  GtkWidget *image107;
  GtkWidget *toolbar1;
  GtkWidget *button_back;
  GtkWidget *image1;
  GtkWidget *button_forward;
  GtkWidget *image2;
  GtkWidget *combo_book;
  GtkWidget *combo_entry_book;
  GtkWidget *combo_chapter;
  GtkWidget *combo_entry_chapter;
  GtkWidget *combo_verse;
  GtkWidget *combo_entry_verse;
  GtkWidget *hbox1;
  GtkWidget *hpaned1;
  GtkWidget *vpaned1;
  GtkWidget *scrolledwindow1;
  GtkWidget *textview_edit;
  GtkWidget *scrolledwindow3;
  GtkWidget *textview_notes;
  GtkWidget *scrolledwindow2;
  GtkWidget *treeview1;
  GtkWidget *statusbar1;
  GtkAccelGroup *accel_group;
  // Gtk variables manually added.
  GtkTextBuffer *textbuffer_edit;
  GtkTextBuffer *textbuffer_notes;
  gint statusbar_context_id;
  GtkListStore *liststore;
  GtkTreeViewColumn *treecolumn;
  GtkTreeSelection *treeselect;
  GtkTextTag *highlighttag_edit;

  /* Initialization */
  virtual void initialize_project ();
  virtual void enable_or_disable_widgets (bool enable);

public:
  /* Menu callbacks */
  virtual void open ();
  virtual void close ();
  virtual void newproject ();
  virtual void editproject ();
  virtual void deleteproject ();
  virtual void convert2web ();
  virtual void viewlog ();
  virtual void showhelp ();
  virtual void showabout ();
  virtual void menu_undo ();
  virtual void menu_redo ();
  virtual void menu_edit ();
  virtual void menu_find ();
  virtual void companion_settings ();
  virtual void editor_settings ();
  virtual void language_settings ();
  virtual void stylesheet_settings ();
  virtual void menu_import ();
protected:

  /* Navigation  */
public:
  virtual void on_combo_book_changed ();
  virtual void on_combo_chapter_changed ();
  virtual void on_combo_verse_changed ();
  virtual void goto_next_verse ();
  virtual void goto_previous_verse ();
  virtual void goto_next_chapter ();
  virtual void goto_previous_chapter ();
  virtual void goto_next_book ();
  virtual void goto_previous_book ();
  virtual void goto_reference ();
protected:
  virtual void go_to_reference (const string & gotobook, const string & gotochapter,
                                const string & gotoverse, bool jump);
  bool go_to_reference_running;
  string editor_current_book, editor_current_chapter, editor_current_verse;
public:
  virtual bool on_navigation_timeout ();
protected:
  bool driving_book, driving_chapter, driving_verse;
public:
  virtual void on_cursor_moved ();
protected:
  bool focus_programmatically_being_grabbed;
public:
  virtual void on_grab_focus ();
protected:
  string cursor_get_verse_number ();
  unsigned int on_grab_focus_timer;
  string goto_cursor_position;
  unsigned int navigation_change_timer;
  string navigator_current_book, navigator_current_chapter, navigator_current_verse;
public:
  bool on_grab_focus_delayed_handler ();
protected:
  unsigned int remove_highlight_timer;

  /* Tracker */
public:
  virtual void goto_next_reference ();
  virtual void goto_previous_reference ();
protected:
  virtual void set_next_or_previous_item (GtkCombo *combo, const string & current_text, bool next);
  vector <string> combo_book_popdown_strings;
  vector <string> combo_chapter_popdown_strings;
  vector <string> combo_verse_popdown_strings;
  virtual void set_sensitivity_of_navigator_buttons ();
public:
  virtual bool on_tracker_timeout ();
protected:
  virtual void set_next_or_previous_reference (bool next);
  Track track;

  /* Variables for general use */
  Configuration* configuration;
  Scripture* scripture;
  Book* navigator_book;

  /* Loading, editing, saving */
  Book* book;
  virtual void editor_load_chapter ();
public:
  virtual bool editor_position_cursor_at_verse ();
protected:
  virtual void editor_save ();
public:
  virtual bool on_save_timeout ();
protected:
  virtual void editor_set_font ();

  /* Clipboard */
public:
  virtual void on_cut ();
  virtual void on_copy ();
  virtual void on_paste ();
protected:

  /* Backups */
//  Backup backup;

  /* List store and reference handling */
  virtual void fill_list_store (vector < string > &lines);
public:
  virtual void on_list_selection_changed ();
  virtual void on_open_references ();
  string selected_reference;
protected:
  virtual void on_save_references ();

  /* Undo and Redo */
  Undo undo;
public:
  virtual void on_insert (GtkTextBuffer *textbuffer, GtkTextIter *pos, const ustring& text, int bytes);
  virtual void on_erase (GtkTextBuffer *textbuffer, GtkTextIter *range_begin, GtkTextIter *range_end);

  /* Undo and Redo */
  virtual bool on_windows_companion_timeout ();
protected:

  /* Bibledit Windows Companion */
  WindowsCompanionLocal windowscompanionlocal;
  WindowsCompanionRemote windowscompanionremote;

  /* Test development */
  virtual void on_test ();
  
};


#endif
